( function( $, settings ) {

	HappyForms.parts = HappyForms.parts || {};

	HappyForms.parts.phone = {
		init: function() {
			this.type = this.$el.data( 'happyforms-type' );
			this.$input = $( 'input', this.$el );
			this.masked = this.$el.attr( 'data-mask' );
			this.prefix = '';

			this.$input.on( 'keyup', this.triggerChange.bind( this ) );
			this.$input.on( 'change', this.triggerChange.bind( this ) );
			this.$input.on( 'blur', this.onBlur.bind(this) );

			if ( this.masked ) {
				this.cleaveInstances = this.$input.map( function( i, input ) {
					var $input = $( input );
					var code = this.$el.attr( 'data-country' );
					var rawValue = $input.val();

					this.prefix = '+' + code;

					var cleave = new Cleave( $input, {
						phone: true,
						phoneRegionCode: settings.codes[code],
						prefix: this.prefix,
					} );

					cleave.setRawValue( rawValue );

					return cleave;
				}.bind( this ) );
			}
		},

		reinit: function() {
			$.each( this.cleaveInstances, function( i, instance ) {
				instance.destroy();
			} );

			this.init();
		},

		isFilled: function() {
			var prefix = this.prefix;

			var filledInputs = this.$input.filter( function() {
				var value = $( this ).val().replace( prefix, '' ).trim();

				return '' !== value;
			} );

			return filledInputs.length > 0;
		},

		onBlur: function() {
			if ( '' !== this.prefix ) {
				return;
			}

			if ( this.$el.is( '.happyforms-part--label-as_placeholder' ) ) {
				if ( this.isFilled() ) {
					this.$el.addClass( 'happyforms-part--filled' );
				} else {
					this.$el.removeClass( 'happyforms-part--filled' );
				}
			}
		},

		serialize: function() {
			var self = this;

			var serialized = this.$input.map( function( i, input ) {
				var $input = $( input );
				var keyValue = {
					name: $input.attr( 'name' ),
					value: $input.val(),
				};

				if ( self.masked ) {
					keyValue.value = self.cleaveInstances[i].getRawValue();
				}

				return keyValue;
			} ).toArray();

			return serialized;
		},
	}

} )( jQuery, HappyFormsPhoneSettings );