( function( $, _, Backbone, api, settings ) {

	var CheckboxModel = happyForms.classes.models.Part.extend( {
		defaults: function() {
			return _.extend(
				{},
				settings.formParts.checkbox.defaults,
				_.result( happyForms.classes.models.Part.prototype, 'defaults' ),
			);
		},

		initialize: function( attrs, options ) {
			happyForms.classes.models.Part.prototype.initialize.apply( this, arguments );

			this.attributes.options = new OptionCollection( this.get( 'options' ), options );
		},

		toJSON: function() {
			var json = Backbone.Model.prototype.toJSON.apply( this, arguments );
			json.options = json.options.toJSON();

			return json;
		},
	} );

	var OptionModel = Backbone.Model.extend( {
		defaults: {
			is_default: false,
			label: '',
		},
	} );

	var OptionCollection = Backbone.Collection.extend( {
		model: OptionModel,
	} );

	var CheckboxView = happyForms.classes.views.Part.extend( {
		template: '#customize-happyforms-checkbox-template',
		itemCustomizeTemplate: '#customize-happyforms-checkbox-item-template',

		initialize: function() {
			happyForms.classes.views.Part.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model.get( 'options' ), 'add change remove', this.onOptionsChange );
			this.listenTo( this, 'ready', this.onReady );
		},

		events: _.extend( {}, happyForms.classes.views.Part.prototype.events, {
			'click .add-option': 'onAddOptionClick',
			'change [name=display_type]': 'onDisplayTypeChange',
			'keyup li [name=label]': 'onCheckboxItemLabelChange',
			'change li [name=is_default]': 'onCheckboxItemDefaultChange',
			'click li .delete-option': 'onDeleteOptionClick',
		} ),

		onReady: function() {
			this.model.get( 'options' ).each( function( itemModel ) {
				this.appendOptionPartial( itemModel );
			}, this );
		},

		onOptionsChange: function() {
			this.model.trigger( 'change' );
		},

		appendOptionPartial: function( itemModel ) {
			var customizeTemplate = _.template( $( this.itemCustomizeTemplate ).html() );
			var customizeHTML = customizeTemplate( { item: itemModel.toJSON() } );

			$( '.options ul', this.$el ).append( customizeHTML );
		},

		onAddOptionClick: function( e ) {
			e.preventDefault();

			var itemID = happyForms.utils.uniqueId( this.model.get( 'id' ) + '_option_', this.model.get( 'options' ) );
			var itemModel = new OptionModel( { id: itemID } );
			this.model.get( 'options' ).add( itemModel );

			this.appendOptionPartial( itemModel );

			var model = this.model;

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onCheckboxItemLabelChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			itemModel.set( 'label', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onCheckboxItemLabelChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onCheckboxItemDefaultChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			if ( $( e.target ).is( ':checked' ) ) {
				itemModel.set( 'is_default', 1 );
			} else {
				itemModel.set( 'is_default', 0 );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onCheckboxItemDefaultChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onDeleteOptionClick: function( e ) {
			e.preventDefault();

			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			this.model.get( 'options' ).remove( itemModel );
			$( '[data-item-id="' + itemModel.get( 'id' ) + '"]', this.$el ).remove();

			if ( this.model.get( 'options' ).length == 0 ) {
				$( '.options ul', this.$el ).html( '' );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onCheckboxItemDeleteCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onDisplayTypeChange: function(e) {
			var data = {
				id: this.model.get( 'id' ),
				callback: 'onCheckboxDisplayTypeChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		}
	} );

	happyForms.previewer = _.extend( happyForms.previewer, {
		onCheckboxDisplayTypeChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );

			if ( 'block' === part.get( 'display_type' ) ) {
				$part.addClass( 'display-type--block' );
			} else {
				$part.removeClass( 'display-type--block' );
			}
		},

		onCheckboxItemLabelChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			$( 'span.label', $option ).text( option.get( 'label' ) );
		},

		onCheckboxItemDefaultChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			$( 'input', $option ).prop( 'checked', option.get( 'is_default' ) );
		},

		onCheckboxItemDeleteCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var $option = $( '#' + options.itemID, $part );

			$option.remove();
		}
	} );

	happyForms.factory.model = _.wrap( happyForms.factory.model, function( func, attrs, options, BaseClass ) {
		if ( 'checkbox' === attrs.type ) {
			BaseClass = CheckboxModel;
		}

		return func( attrs, options, BaseClass );
	} );

	happyForms.factory.view = _.wrap( happyForms.factory.view, function( func, options, BaseClass ) {
		if ( 'checkbox' === options.type ) {
			BaseClass = CheckboxView;
		}

		return func( options, BaseClass );
	} );

} ) ( jQuery, _, Backbone, wp.customize, _happyFormsSettings );
