( function( $, _, Backbone, api, settings ) {

	var RadioModel = happyForms.classes.models.Part.extend( {
		defaults: function() {
			return _.extend(
				{},
				settings.formParts.radio.defaults,
				_.result( happyForms.classes.models.Part.prototype, 'defaults' ),
			);
		},

		initialize: function( attrs, options ) {
			happyForms.classes.models.Part.prototype.initialize.apply( this, arguments );

			this.attributes.options = new OptionCollection( this.get( 'options' ), options );
		},

		toJSON: function() {
			var json = Backbone.Model.prototype.toJSON.apply( this, arguments );
			json.options = json.options.toJSON();

			return json;
		},
	} );

	var OptionModel = Backbone.Model.extend( {
		defaults: {
			is_default: false,
			label: '',
			description: '',
		},
	} );

	var OptionCollection = Backbone.Collection.extend( {
		model: OptionModel,
	} );

	var RadioView = happyForms.classes.views.Part.extend( {
		template: '#customize-happyforms-radio-template',
		itemCustomizeTemplate: '#customize-happyforms-radio-item-template',

		initialize: function() {
			happyForms.classes.views.Part.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model.get( 'options' ), 'add change remove', this.onOptionsChange );
			this.listenTo( this, 'ready', this.onReady );
		},

		events: _.extend( {}, happyForms.classes.views.Part.prototype.events, {
			'click .add-option': 'onAddOptionClick',
			'click .delete-option': 'onDeleteOptionClick',
			'keyup li [name=option_label]': 'onItemLabelChange',
			'keyup li [name=option_description]': 'onItemDescriptionChange',
			'change li [name=option_is_default]': 'onItemDefaultChange',
			'change [name=display_type]': 'onDisplayTypeChange'
		} ),

		onReady: function() {
			this.model.get( 'options' ).each( function( itemModel ) {
				this.appendOptionPartial( itemModel );
			}, this );
		},

		onOptionsChange: function() {
			this.model.trigger( 'change' );
		},

		appendOptionPartial: function( itemModel ) {
			var customizeTemplate = _.template( $( this.itemCustomizeTemplate ).html() );
			var customizeHTML = customizeTemplate( { item: itemModel.toJSON() } );

			$( '.options ul', this.$el ).append( customizeHTML );
		},

		onAddOptionClick: function( e ) {
			e.preventDefault();

			var itemID = happyForms.utils.uniqueId( this.model.get( 'id' ) + '_option_', this.model.get( 'options' ) );
			var itemModel = new OptionModel( { id: itemID } );
			this.model.get( 'options' ).add( itemModel );

			this.appendOptionPartial( itemModel );

			var model = this.model;

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onDeleteOptionClick: function( e ) {
			e.preventDefault();

			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			this.model.get( 'options' ).remove( itemModel );
			$( '[data-item-id="' + itemID + '"]', this.$el ).remove();

			if ( this.model.get( 'options' ).length == 0 ) {
				$( '.options ul', this.$el ).html( '' );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onRadioItemDeleteCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onItemLabelChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			itemModel.set( 'label', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onRadioItemLabelChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onItemDescriptionChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			itemModel.set( 'description', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onRadioItemDescriptionChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onItemDefaultChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );
			var isChecked = $( e.target ).is( ':checked' );

			this.model.get( 'options' ).forEach( function( item ) {
				item.set( 'is_default', 0 );
			} );

			$( 'li [name=option_is_default]', this.$el ).attr( 'checked', false );

			if ( isChecked ) {
				itemModel.set( 'is_default', 1 );
				$( e.target ).attr( 'checked', true );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onRadioItemDefaultChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onDisplayTypeChange: function(e) {
			var $input = $( e.target );
			var attribute = $input.data( 'bind' );

			this.model.set( attribute, $input.val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onRadioDisplayTypeChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		}
	} );

	happyForms.previewer = _.extend( happyForms.previewer, {
		onRadioItemDeleteCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var $option = $( '#' + options.itemID, $part );

			$option.remove();
		},

		onRadioDisplayTypeChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );

			if ( 'block' === part.get( 'display_type' ) ) {
				$part.addClass( 'display-type--block' );
			} else {
				$part.removeClass( 'display-type--block' );
			}
		},

		onRadioItemLabelChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			this.$( 'span.label', $option ).text( option.get( 'label' ) );
		},

		onRadioItemDescriptionChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			this.$( '.happyforms-part-option__description', $option ).text( option.get( 'description' ) );
		},

		onRadioItemDefaultChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			this.$( 'input', $option ).prop( 'checked', option.get( 'is_default' ) );
		},
	} );

	happyForms.factory.model = _.wrap( happyForms.factory.model, function( func, attrs, options, BaseClass ) {
		if ( 'radio' === attrs.type ) {
			BaseClass = RadioModel;
		}

		return func( attrs, options, BaseClass );
	} );

	happyForms.factory.view = _.wrap( happyForms.factory.view, function( func, options, BaseClass ) {
		if ( 'radio' === options.type ) {
			BaseClass = RadioView;
		}

		return func( options, BaseClass );
	} );

} ) ( jQuery, _, Backbone, wp.customize, _happyFormsSettings );
