( function( $, _, Backbone, api, settings ) {

	var SelectModel = happyForms.classes.models.Part.extend( {
		defaults: function() {
			return _.extend(
				{},
				settings.formParts.select.defaults,
				_.result( happyForms.classes.models.Part.prototype, 'defaults' ),
			);
		},

		initialize: function( attrs, options ) {
			happyForms.classes.models.Part.prototype.initialize.apply( this, arguments );

			this.attributes.options = new OptionCollection( this.get( 'options' ), options );
		},

		toJSON: function() {
			var json = Backbone.Model.prototype.toJSON.apply( this, arguments );
			json.options = json.options.toJSON();

			return json;
		},
	} );

	var OptionModel = Backbone.Model.extend( {
		defaults: {
			is_default: false,
			label: '',
		},
	} );

	var OptionCollection = Backbone.Collection.extend( {
		model: OptionModel,
	} );

	var SelectView = happyForms.classes.views.Part.extend( {
		template: '#customize-happyforms-select-template',
		itemCustomizeTemplate: '#customize-happyforms-select-item-template',

		initialize: function() {
			happyForms.classes.views.Part.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model.get( 'options' ), 'add change remove', this.onOptionsChange );
			this.listenTo( this, 'ready', this.onReady );
		},

		events: _.extend( {}, happyForms.classes.views.Part.prototype.events, {
			'click .add-option': 'onAddOptionClick',
			'click .show-all-options': 'onShowAllOptionsClick',
			'keyup li [name=label]': 'onSelectItemLabelChange',
			'change li [name=is_default]': 'onSelectItemDefaultChange',
			'click li .delete-option': 'onDeleteOptionClick',
		} ),

		onReady: function() {
			this.model.get( 'options' ).each( function( itemModel, index ) {
				this.appendOptionPartial( itemModel, index, this.model.get('options').length );
			}, this );
		},

		onOptionsChange: function() {
			this.model.trigger( 'change' );
		},

		appendOptionPartial: function( itemModel, index, totalItems ) {
			if ( 'undefined' === typeof index ) {
				index = false;
			}

			if ( 'undefined' === typeof totalItems ) {
				totalItems = false;
			}

			var customizeTemplate = _.template( $( this.itemCustomizeTemplate ).html() );
			var customizeHTML = customizeTemplate( { item: itemModel.toJSON() } );

			$( '.options ul', this.$el ).append( customizeHTML );

			if (index && index > 4 && index < totalItems) {
				this.$el.find('[data-item-id='+ itemModel.id +']').hide();

				this.$el.find('.show-all-options span').text(totalItems);
				this.$el.find('.show-all-options').show();
			}
		},

		onAddOptionClick: function( e ) {
			e.preventDefault();

			var itemID = happyForms.utils.uniqueId( this.model.get( 'id' ) + '_option_', this.model.get( 'options' ) );
			var itemModel = new OptionModel( { id: itemID } );
			this.model.get( 'options' ).add( itemModel );

			this.appendOptionPartial( itemModel );

			var model = this.model;

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onShowAllOptionsClick: function(e) {
			var $link = $(e.target);

			this.$el.find('.happyforms-part-widget--sub').show();

			$link.hide();
		},

		onSelectItemLabelChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			itemModel.set( 'label', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onSelectItemLabelChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onSelectItemDefaultChange: function( e ) {
			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );
			var checked = $( e.target ).is( ':checked' );

			$( '[name=is_default]', this.$el ).attr( 'checked', false );

			this.model.get( 'options' ).each( function( itemModel ) {
				itemModel.set( 'is_default', 0 );
			} );

			if ( checked ) {
				$( e.target ).attr( 'checked', true );
				itemModel.set( 'is_default', 1 );
			} else {
				itemModel.set( 'is_default', 0 );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onSelectItemDefaultChangeCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onDeleteOptionClick: function( e ) {
			e.preventDefault();

			var itemID = $( e.target ).parents( '[data-item-id]' ).data( 'item-id' );
			var itemModel = this.model.get( 'options' ).get( itemID );

			this.model.get( 'options' ).remove( itemModel );
			$( '[data-item-id="' + itemModel.get( 'id' ) + '"]', this.$el ).remove();

			if ( this.model.get( 'options' ).length == 0 ) {
				$( '.options ul', this.$el ).html( '' );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onSelectItemDeleteCallback',
				options: {
					itemID: itemID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},
	} );

	happyForms.previewer = _.extend( happyForms.previewer, {
		onSelectItemAdd: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );

			this.$( '.happyforms-part__el', $part ).append( this.$( options.optionHTML ) );
		},

		onSelectItemLabelChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			$option.text( option.get( 'label' ) );
		},

		onSelectItemDefaultChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var option = part.get( 'options' ).get( options.itemID );
			var $option = $( '#' + options.itemID, $part );

			$option.attr( 'selected', true );
		},

		onSelectItemDeleteCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var $option = $( '#' + options.itemID, $part );

			$option.remove();
		}
	} );

	happyForms.factory.model = _.wrap( happyForms.factory.model, function( func, attrs, options, BaseClass ) {
		if ( 'select' === attrs.type ) {
			BaseClass = SelectModel;
		}

		return func( attrs, options, BaseClass );
	} );

	happyForms.factory.view = _.wrap( happyForms.factory.view, function( func, options, BaseClass ) {
		if ( 'select' === options.type ) {
			BaseClass = SelectView;
		}

		return func( options, BaseClass );
	} );

} ) ( jQuery, _, Backbone, wp.customize, _happyFormsSettings );
