( function( $, _, Backbone, api, settings, partSettings ) {

	var TableModel = happyForms.classes.models.Part.extend( {
		defaults: function() {
			return _.extend(
				{},
				settings.formParts.table.defaults,
				_.result( happyForms.classes.models.Part.prototype, 'defaults' ),
			);
		},

		initialize: function( attrs, options ) {
			happyForms.classes.models.Part.prototype.initialize.apply( this, arguments );

			this.attributes.columns = new ColumnCollection( this.get( 'columns' ), options );
			this.attributes.rows = new RowCollection( this.get( 'rows' ), options );
		},

		toJSON: function() {
			var json = Backbone.Model.prototype.toJSON.apply( this, arguments );
			json.columns = json.columns.toJSON();
			json.rows = json.rows.toJSON();

			return json;
		},
	} );

	var ColumnModel = Backbone.Model.extend( {
		defaults: partSettings.column,
	} );

	var ColumnCollection = Backbone.Collection.extend( {
		model: ColumnModel,
	} );

	var RowModel = Backbone.Model.extend( {
		defaults: partSettings.row,
	} );

	var RowCollection = Backbone.Collection.extend( {
		model: RowModel,
	} );

	var TableView = happyForms.classes.views.Part.extend( {
		template: '#customize-happyforms-table-template',
		columnCustomizeTemplate: '#customize-happyforms-table-column-template',
		rowCustomizeTemplate: '#customize-happyforms-table-row-template',

		initialize: function() {
			happyForms.classes.views.Part.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model, 'change:allow_multiple_selection', this.onMultipleSelectionChange );
			this.listenTo( this.model.get( 'columns' ), 'add change remove', this.onColumnsChange );
			this.listenTo( this.model.get( 'rows' ), 'add change remove', this.onRowsChange );
			this.listenTo( this, 'ready', this.onReady );
		},

		events: _.extend( {}, happyForms.classes.views.Part.prototype.events, {
			'click [data-happyforms-tab]': 'onTabClick',
			'click .add-column': 'onAddColumnClick',
			'click .delete-column': 'onDeleteColumnClick',
			'keyup li [name=column_label]': 'onColumnLabelChange',
			'change li [name=column_is_default]': 'onColumnDefaultChange',
			'click .add-row': 'onAddRowClick',
			'click .delete-row': 'onDeleteRowClick',
			'keyup li [name=row_label]': 'onRowLabelChange',
		} ),

		onReady: function() {
			this.model.get( 'columns' ).each( function( columnModel ) {
				this.appendColumnPartial( columnModel );
			}, this );

			this.model.get( 'rows' ).each( function( rowModel ) {
				this.appendRowPartial( rowModel );
			}, this );
		},

		onMultipleSelectionChange: function() {
			var model = this.model;

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onColumnsChange: function() {
			this.model.trigger( 'change' );
		},

		onRowsChange: function() {
			this.model.trigger( 'change' );
		},

		onTabClick: function( e ) {
			var $link = $( e.target );
			var $links = $link.siblings();
			var tabClass = $link.attr( 'data-happyforms-tab' );
			var $tabs = $( '.tab-content > div', this.$el );
			var $tab = $( '.tab-content > .' + tabClass, this.$el );

			$links.removeClass( 'active' );
			$link.addClass( 'active' );
			$tabs.removeClass( 'active' );
			$tab.addClass( 'active' );
		},

		appendColumnPartial: function( columnModel ) {
			var customizeTemplate = _.template( $( this.columnCustomizeTemplate ).html() );
			var customizeHTML = customizeTemplate( { column: columnModel.toJSON() } );

			$( '.options.columns ul', this.$el ).append( customizeHTML );
		},

		appendRowPartial: function( rowModel ) {
			var customizeTemplate = _.template( $( this.rowCustomizeTemplate ).html() );
			var customizeHTML = customizeTemplate( { row: rowModel.toJSON() } );

			$( '.options.rows ul', this.$el ).append( customizeHTML );
		},

		onAddColumnClick: function( e ) {
			e.preventDefault();

			var columnID = happyForms.utils.uniqueId( this.model.get( 'id' ) + '_column_', this.model.get( 'columns' ) );
			var columnModel = new ColumnModel( { id: columnID } );
			var model = this.model;

			this.model.get( 'columns' ).add( columnModel );
			this.appendColumnPartial( columnModel );

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onDeleteColumnClick: function( e ) {
			e.preventDefault();

			var columnID = $( e.target ).parents( '[data-column-id]' ).data( 'column-id' );
			var columnModel = this.model.get( 'columns' ).get( columnID );

			this.model.get( 'columns' ).remove( columnModel );
			$( '[data-column-id="' + columnID + '"]', this.$el ).remove();

			if ( this.model.get( 'columns' ).length == 0 ) {
				$( '.options.columns ul', this.$el ).html( '' );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onTableColumnDeleteCallback',
				options: {
					columnID: columnID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onColumnLabelChange: function( e ) {
			var columnID = $( e.target ).parents( '[data-column-id]' ).data( 'column-id' );
			var columnModel = this.model.get( 'columns' ).get( columnID );

			columnModel.set( 'label', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onTableColumnLabelChangeCallback',
				options: {
					columnID: columnID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onColumnDefaultChange: function( e ) {
			var columnID = $( e.target ).parents( '[data-column-id]' ).data( 'column-id' );
			var columnModel = this.model.get( 'columns' ).get( columnID );
			var isChecked = $( e.target ).is( ':checked' );

			this.model.get( 'columns' ).forEach( function( column ) {
				column.set( 'is_default', 0 );
			} );

			$( 'li [name=column_is_default]', this.$el ).attr( 'checked', false );

			if ( isChecked ) {
				columnModel.set( 'is_default', 1 );
				$( e.target ).attr( 'checked', true );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onTableColumnDefaultChangeCallback',
				options: {
					columnID: columnID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onAddRowClick: function( e ) {
			e.preventDefault();

			var rowID = happyForms.utils.uniqueId( this.model.get( 'id' ) + '_row_', this.model.get( 'rows' ) );
			var rowModel = new ColumnModel( { id: rowID } );
			var model = this.model;

			this.model.get( 'rows' ).add( rowModel );
			this.appendRowPartial( rowModel );

			this.model.fetchHtml( function( response ) {
				var data = {
					id: model.get( 'id' ),
					html: response,
				};

				api.previewer.send( 'happyforms-form-part-refresh', data );
			} );
		},

		onDeleteRowClick: function( e ) {
			e.preventDefault();

			var rowID = $( e.target ).parents( '[data-row-id]' ).data( 'row-id' );
			var rowModel = this.model.get( 'rows' ).get( rowID );

			this.model.get( 'rows' ).remove( rowModel );
			$( '[data-row-id="' + rowID + '"]', this.$el ).remove();

			if ( this.model.get( 'rows' ).length == 0 ) {
				$( '.options.rows ul', this.$el ).html( '' );
			}

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onTableRowDeleteCallback',
				options: {
					rowID: rowID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		onRowLabelChange: function( e ) {
			var rowID = $( e.target ).parents( '[data-row-id]' ).data( 'row-id' );
			var rowModel = this.model.get( 'rows' ).get( rowID );

			rowModel.set( 'label', $( e.target ).val() );

			var data = {
				id: this.model.get( 'id' ),
				callback: 'onTableRowLabelChangeCallback',
				options: {
					rowID: rowID,
				}
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},
	} );

	happyForms.previewer = _.extend( happyForms.previewer, {

		onTableColumnDeleteCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var $column = $( '#' + options.columnID, $part );
			var columnIndex = $column.index() + 1;

			$column.remove();
			this.$( 'td:nth-child(' + columnIndex + ')', $part ).remove();
		},

		onTableColumnLabelChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var column = part.get( 'columns' ).get( options.columnID );
			var $column = $( '#' + options.columnID, $part );

			this.$( 'span', $column ).text( column.get( 'label' ) );
		},

		onTableColumnDescriptionChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var column = part.get( 'columns' ).get( options.columnID );
			var $column = $( '#' + options.columnID, $part );

			this.$( '.happyforms-part-option__description', $column ).text( column.get( 'description' ) );
		},

		onTableRowDeleteCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var $row = $( '#' + options.rowID, $part );

			$row.remove();
		},

		onTableRowDescriptionChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var row = part.get( 'rows' ).get( options.rowID );
			var $row = $( '#' + options.rowID, $part );

			this.$( '.happyforms-part-option__description', $row ).text( row.get( 'description' ) );
		},

		onTableRowLabelChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var row = part.get( 'rows' ).get( options.rowID );
			var $row = $( '#' + options.rowID, $part );

			this.$( '.happyforms-table__row-label', $row ).text( row.get('label') );
		},

		onTableColumnDefaultChangeCallback: function( id, html, options ) {
			var part = this.getPartModel( id );
			var $part = this.getPartElement( html );
			var column = part.get( 'columns' ).get( options.columnID );
			var $column = $( '#' + options.columnID, $part );
			var columnIndex = $column.index() + 1;
			var isDefault = column.get( 'is_default' );

			this.$( '.happyforms-table__row--body .happyforms-table__cell:nth-child(' + columnIndex + ') input', $part ).prop( 'checked', isDefault );
		},
	} );

	happyForms.factory.model = _.wrap( happyForms.factory.model, function( func, attrs, options, BaseClass ) {
		if ( 'table' === attrs.type ) {
			BaseClass = TableModel;
		}

		return func( attrs, options, BaseClass );
	} );

	happyForms.factory.view = _.wrap( happyForms.factory.view, function( func, options, BaseClass ) {
		if ( 'table' === options.type ) {
			BaseClass = TableView;
		}

		return func( options, BaseClass );
	} );

} ) ( jQuery, _, Backbone, wp.customize, _happyFormsSettings, _happyFormsTableSettings );
