<?php

class HappyForms_Part_Title extends HappyForms_Form_Part {

	public $type = 'title';

	public function __construct() {
		$this->label = __( 'Title', 'happyforms' );
		$this->description = __( 'For displaying personal honorifics.', 'happyforms' );
	}

	/**
	 * Get all part meta fields defaults.
	 *
	 * @since 1.0.0.
	 *
	 * @return array
	 */
	public function get_customize_fields() {
		return array(
			'type' => array(
				'default' => $this->type,
				'sanitize' => 'sanitize_text_field',
			),
			'label' => array(
				'default' => __( 'Title', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'label_placement' => array(
				'default' => 'above',
				'sanitize' => 'sanitize_text_field'
			),
			'description' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'tooltip_description' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'width' => array(
				'default' => 'full',
				'sanitize' => 'sanitize_key'
			),
			'css_class' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'required' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			)
		);
	}

	/**
	 * Get template for part item in customize pane.
	 *
	 * @since 1.0.0.
	 *
	 * @return string
	 */
	public function customize_templates() {
		require_once( happyforms_get_include_folder() . '/templates/parts/customize-title.php' );
	}

	/**
	 * Get front end part template with parsed data.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	$part_data 	Form part data.
	 * @param array	$form_data	Form (post) data.
	 *
	 * @return string	Markup for the form part.
	 */
	public function frontend_template( $part_data = array(), $form_data = array() ) {
		$part = wp_parse_args( $part_data, $this->get_customize_defaults() );
		$part['options'] = $this->get_honorifics();
		$form = $form_data;

		include( happyforms_get_include_folder() . '/templates/parts/frontend-title.php' );
    }

    /**
     * Get honorifics as possible options for the dropdown.
     *
     * @since 1.1.0.
     *
     * @return array List of honorifics.
     */
    public function get_honorifics() {
        return array(
            array( 'id' => 'mr', 'label' => __( 'Mr.', 'happyforms' ) ),
            array( 'id' => 'mrs', 'label' => __( 'Mrs.', 'happyforms' ) ),
            array( 'id' => 'ms', 'label' => __( 'Ms.', 'happyforms' ) ),
            array( 'id' => 'miss', 'label' => __( 'Miss', 'happyforms' ) ),
            array( 'id' => 'prof', 'label' => __( 'Prof.', 'happyforms' ) ),
            array( 'id' => 'dr', 'label' => __( 'Dr.', 'happyforms' ) ),
        );
    }

	/**
	 * Enqueue scripts in customizer area.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	List of dependencies.
	 *
	 * @return void
	 */
	public function customize_enqueue_scripts( $deps = array() ) {
		wp_enqueue_script(
			'part-title',
			happyforms_get_plugin_url() . 'assets/js/parts/part-title.js',
			$deps, HAPPYFORMS_VERSION, true
		);
	}

	/**
	 * Sanitize submitted value before storing it.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part_data Form part data.
	 *
	 * @return string
	 */
	public function sanitize_value( $part_data = array(), $form_data = array() ) {
		$sanitized_value = $this->get_default_value();
		$part_name = happyforms_get_part_name( $part_data, $form_data );

		if ( isset( $_REQUEST[$part_name] ) ) {
			$sanitized_value = sanitize_text_field( $_REQUEST[$part_name] );

			return $sanitized_value;
		}
	}

	/**
	 * Validate value before submitting it. If it fails validation, return WP_Error object, showing respective error message.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part Form part data.
	 * @param string $value Submitted value.
	 *
	 * @return string|object
	 */
	public function validate_value( $value, $part = array(), $form = array() ) {
		$validated_value = $value;

		if ( 1 === $part['required'] && empty( $validated_value ) ) {
			return new WP_Error( 'error', __( 'This field is required.', 'happyforms' ) );
		}

		$options = happyforms_get_part_options( $this->get_honorifics(), $part, $form );
		$options = wp_list_pluck( $options, 'label' );

		if ( ! in_array( $validated_value, $options ) ) {
			return new WP_Error( 'error', __( 'Values is not valid.', 'happyforms' ) );
		}

		return $validated_value;
	}

}
